//	NPBPlugin.h - Vst plugin object, derived from NPBPluginBase.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBPLUGIN_H_
#define NPBPLUGIN_H_

#include "aeffectx.h"

//because we use it in NPBPluginEditor.h
class NPBPlugin;

#include "NPBPluginBase.h"
#include "NPBPluginEditor.h"
#include "NPBPluginGUI.h"

#include "NPBMapping.h"

///	Okay, I'm doing this because I can't for the life of me figure out how to cast from (VstEvent *[250]) to (VstEvent *[2])...
struct NPBVstEvents
{
	long numEvents;
	long reserved;
	VstEvent* events[250];
};

///	Class encapsulating a VST plugin.
class NPBPlugin : public NPBPluginBase,
						 ActionListener,
						 Timer
{
  public:
	///	Constructor.
	/*!
		This version opens a file dialog to allow the user to select the
		plugin.

		This is the version used most of the time, and the only one that can
		be used with the factory.
	 */
	NPBPlugin(NPBPluginOrganiser *parent);
	///	Constructor.
	/*!
		\param filepath Path to the VST plugin to use.

		This one is used when a VST plugin dll is dropped on the
		NPBPluginOrganiser (i.e. the filepath is passed in - it wouldn't make
		sense to open a file dialog in this situation).  It's also used when
		you load a plugin patch/configuration, for the same reason.
	 */
	NPBPlugin(NPBPluginOrganiser *parent, String filepath);
	///	Destructor.
	~NPBPlugin();

	///	Draws the background, using the parent NPBPluginOrganiser's pluginColour.
	/*!
		Also draws the name of the plugin in the centre.
	 */
	void paint(Graphics& g);
	///	Used to react when the buttons are pressed.
	void actionListenerCallback(const String& m);
	///	Not used?
	void timerCallback();

	///	Passes the input audio to the plugin to be processed and placed in output.
	/*!
		MIDI events are also passed to the plugin at the start of this method.
	 */
	void ProcessAudio(float **input, float **output, int numsamples);

	///	Informing the plugin that the audio has just started playing.
	void StartAudio();
	///	Informing the plugin that the audio has been stopped.
	void StopAudio();

	///	To position the various Components correctly.
	void resized();
	///	To detect when the right mouse button menu should be displayed.
	void mouseDown(const MouseEvent& e);

	///	Not used?
	void vstIdle();

	///	Returns the value of the Mix parameter.
	/*!
		The Mix parameter is used to set the ratio of wet to dry signal sent
		to the output.
	 */
	float getMix() {return Mix;};
	///	Sets the Mix parameter.
	/*!
		The Mix parameter is used to set the ratio of wet to dry signal sent
		to the output.
	 */
	void setMix(float val) {Mix = val;};
	///	Used by NPBPluginGUI to inform NPBPlugin that the gui no longer exists (because the user's closed it).
	void setGUI(NPBPluginGUI *val) {GUI = val;};
	///	Used by NPBPluginEditor to inform NPBPlugin that the editor no longer exists (because the user's closed it).
	void setEditor(NPBPluginEditor *val) {Editor = val;};
	///	Returns true if the VST plugin exists.
	/*!
		Obviously this will return false if something went wrong.
	 */
	bool getIsOk() {if(Plugin) return true; else return false;};

	///	Sets the NPBMapping array of mappings.
	/*!
		Called by NPBMappingsDialog to update the plugin's mappings after the
		user's changed them.
	 */
	void setMappings(OwnedArray<NPBMapping> *array);
	///	Returns the name of the indexed parameter.
	/*!
		Called by NPBMappingsDialog.
	 */
	void getParameterName(long index, String& name);
	///	Returns the number of parameters the plugin has.
	/*!
		Doesn't include the Mix and Bypass parameters we give all plugins.

		Called by NPBMappingsDialog.
	 */
	long getNumParameters_vst() {return Plugin->numParams;};
	///	Returns the number of joysticks attached to the system.
	int getNumJoysticks();

	///	Called when a joystick event happens.
	/*!
		It's in this method that we send parameter/MIDI messages to the plugin
		when the joystick moves/changes state.
	 */
	void joystickCallback(JoystickEvent& joystate, int index);

	///	Returns the index of the current VST plugin program.
	/*!
		Called by NPBPluginOrganiser when it's saving the current plugin
		patch/configuration.
	 */
	long getProgramNum() {char tempstr[256]; return Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f);};
	///	Returns the total number of parameters for the plugin.
	/*!
		Includes the Bypass and Mix parameters we provide for all plugins.

		Called by NPBPluginOrganiser when it's saving the current plugin
		patch/configuration.
	 */
	long getNumParameters() {return Plugin->numParams+2;};
	///	Returns the value of the indexed parameter.
	/*!
		Called by NPBPluginOrganiser when it's saving the current plugin
		patch/configuration.
	 */
	float getParameter(long index);
	///	Returns the path to the VST dll.
	/*!
		Called by NPBPluginOrganiser when it's saving the current plugin
		patch/configuration.
	 */
	String getFile() {return FilePath;};
	///	Returns the number of mappings this plugin has.
	/*!
		Called by NPBPluginOrganiser when it's saving the current plugin
		patch/configuration.
	 */
	int getNumMappings() {return MappingsArray.size();};
	///	Returns a pointer to the indexed mapping.
	/*!
		Called by NPBPluginOrganiser when it's saving the current plugin
		patch/configuration.
	 */
	NPBMapping *getMapping(int index) {return MappingsArray[index];};
	///	Sets the current program of the VST plugin.
	/*!
		Called by NPBPluginOrganiser when it's loading a new plugin
		patch/configuration.
	 */
	void setProgramNum(long index) {char tempstr[256]; Plugin->dispatcher(Plugin, effSetProgram, 0, index, tempstr, 0.0f);};
	///	Sets the indexed parameter to val.
	/*!
		Called by NPBPluginOrganiser when it's loading a new plugin
		patch/configuration.
	 */
	void setParameter(long index, float val);
	///	Adds map to the plugin's array of mappings.
	/*!
		Called by NPBPluginOrganiser when it's loading a new plugin
		patch/configuration.
	 */
	void setMapping(NPBMapping *map) {MappingsArray.add(map);};

	///	Called from NPBPluginOrganiser.
	/*!
		Sets how much the envelope for the envelope detector decrements by
		when the input signal is lower than the envelope value.
	 */
	void setEnvelopeDecrement(float val) {envelopeDecrement = val;};

	///	Called from the organiser in the vst callback, to tell us the plugin requires MIDI events.
	AEffect *getActualPlugin() {return Plugin;};
	///	Called from the vst callback to tell us the plugin requires MIDI events.
	void setWantsEvents() {wantsEvents = true;};

	///	Called from the midi input callback in the Organiser, to pass midi messages from the MIDI input device on to us.
	void addMIDIMessage(const MidiMessage& message);

	///	Factory to construct a new instance of this plugin.
	static Factory<NPBPluginBase, NPBPlugin> myFactory;

	///	Hmm...  This doesn't look good...
	bool EditorOpen;
  private:
	///	Handles the loading of the VST plugin dll in plugin.
	void LoadPlugin(File plugin);

	///	In ProcessAudio(), we may have mappings to deal with from the envelope detector.
	/*!
		This just makes the ProcessAudio() code a little tidier.

		\param mappingIndex The mapping in MappingsArray we're dealing with.
		\param val The value to send to the plugin.
	 */
	void handleMapping(long mappingIndex, float val);

	TextButton *Edit;
	TextButton *Mappings;
	TextButton *Bypass;

	String PluginName;
	float Mix;

	///	Pointer to the actual dll.
	void *PluginHandle;
	///	Pointer to the plugin object itself.
	AEffect *Plugin;
	///	Path to the plugin dll.
	String FilePath;

	///	If the plugin doesn't have it's own gui, provides sliders etc. instead.
	NPBPluginEditor *Editor;
	///	If the plugin *does* have it's own gui.
	NPBPluginGUI *GUI;

	///	The array of mappings for this plugin.
	OwnedArray<NPBMapping> MappingsArray;

	///	Envelope value for the envelope detector.
	float envelope;
	///	How much the envelope for the envelope detector is decremented by when the input is lower than it's current value.
	float envelopeDecrement;

	///	This represents the current MIDI note.
	/*!
		If we send a MIDI noteOn message to the plugin, it's done by sending
		a velocity message > 0.  We therefore need to keep a note of what the
		actual NoteVal is.
	 */
	int midiNoteVal;
	///	This is to keep note of whether we should ignore all further Velocity messages, until we get one set to 0.
	bool noteOn[128];
	///	Whether the plugin wants MIDI events or not.
	bool wantsEvents;

	///	MIDI events sent to the plugin.
	/*!
		The way this works is we fill it with events when the joystick sends
		them, then at the start of each audio processing block, we pass a
		pointer to this structure to the plugin via processEvents().  At the
		end of the block, we indicate that we're finished with the events in
		the block.
	 */
	NPBVstEvents events;
	///	This is an array of MIDI events we keep to avoid having to keep allocating & de-allocating memory for them.
	VstEvent mainEvents[250];
	///	This is an array keeping track of which events in mainEvents are pending.
	bool pendingEvents[250];
	///	This is how many events we have pending.
	int numPendingEvents;
};

#endif
