//	NPBPluginBase.h - Handles all the methods used to access plugins.
//					  Currently has a VST subclass, and a simple gain
//					  subclass.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBPLUGINBASE_H_
#define NPBPLUGINBASE_H_

#ifndef NPBPLUGINORGANISER_H_
#include "NPBPluginOrganiser.h"
#endif

#include <juce.h>

#include "NPBPluginFactory.h"

#ifndef NPBMAPPING_H_
#include "NPBMapping.h"
#endif

enum
{
	DELETE_THIS_PLUGIN,
};

struct AEffect;

///	This is the base class for any kind of audio altering object (I've confusingly termed them plugins) which can reside within the Organiser.
/*!
	Basically a class derived from JUCE's Component, with various
	audio-related methods.  The reasoning behind doing this instead of just
	using VST plugins for everything was so I could integrate GUI components
	in with the main program.

	\sa NPBPluginOrganiser
   */
class NPBPluginBase : public Component
{
  public:
	///	Constructor.
	NPBPluginBase(NPBPluginOrganiser *parent);
	///	Destructor.
	virtual ~NPBPluginBase();

	///	Sets the ID of the plugin class.
	/*!
		\sa NPBPluginBase::ID
	 */
	void setID(int id) {ID = id;};
	///	Returns this plugin class' ID.
	/*!
		\sa NPBPluginBase::ID
	 */
	int getID() {return ID;};

	///	Give the plugin a pointer to the parent NPBPluginOrganiser.
	void setParent(NPBPluginOrganiser *parent) {Parent = parent;};

	//should be pure virtual, but can't be due to Array restrictions
	///	Processes a block of samples, to send to the soundcard o/p.
	/*!
		This should really be pure virtual, but array restrictions prevent
		that.
	 */
	virtual void ProcessAudio(float **input, float **output, int numsamples) {};

	///	Informs the plugin that audio processing has started.
	virtual void StartAudio() {};
	///	Informs the plugin that audio processing has stopped.
	virtual void StopAudio() {};

	///	Not used.
	/*!
		\todo Get a proper object-oriented way of handling these kind of
		VST-specific things.
	 */
	virtual void vstCalls(String callname, long index, long value, void *ptr, float opt) {};

	///	Called after the plugin is constructed, to check that it loaded okay.
	virtual bool getIsOk() {return true;};
	///	Returns true if the plugin needs to be deleted.
	/*!
		This is called from NPBPluginOrganiser, after it's received a
		DELETE_THIS_PLUGIN CommandMessage, so the Organiser can determine
		which plugin is supposed to be deleted.
	 */
	bool getDeleted() {return NeedsDeleted;};
	///	Informs the plugin how many channels it's supposed to be working with.
	virtual void setNumChannels(int num) {NumChannels = num;};

	///	Hmm...
	/*!
		Why the hell have I got an Organiser and a Parent???

		\sa NPBPluginBase::setParent
	 */
	void setOrganiser(NPBPluginOrganiser *organise) {Organiser = organise;};

	///	Called when the program's received a joystick event.
	/*!
		This is so joystick events can be passed to plugins, to handle
		mappings between joystick controls and parameters, MIDI etc.
	 */
	virtual void joystickCallback(JoystickEvent& joystate, int index) {};

	///	Xml method (for loading and saving patches).
	virtual long getProgramNum() {return 0;};
	///	Xml method (for loading and saving patches).
	virtual void setProgramNum(long index) {};
	///	Xml method (for loading and saving patches).
	virtual long getNumParameters() {return 0;};
	///	Xml method (for loading and saving patches).
	virtual float getParameter(long index) {return 0.0f;};
	///	Xml method (for loading and saving patches).
	virtual void setParameter(long index, float val) {};
	///	Xml method (for loading and saving patches).
	/*!
		This is used in NPBPlugin, to store the path to the actual plugin dll.
	 */
	virtual String getFile() {return "nope";};
	///	Xml method (for loading and saving patches).
	virtual int getNumMappings() {return 0;};
	///	Xml method (for loading and saving patches).
	virtual NPBMapping *getMapping(int index) {return 0;};
	///	Xml method (for loading and saving patches).
	virtual void setMapping(NPBMapping *map) {};

	///	Called from the organiser in the vst callback, to tell us the plugin requires MIDI events.
	/*!
		Only relevant for NPBPlugin (this is a pretty nasty thing to do
		to the base class, but I can't think of another way right now).
	 */
	virtual AEffect *getActualPlugin() {return 0;};

	///	Called from the midi input callback in the Organiser, to pass midi messages from the MIDI input device on to us.
	virtual void addMIDIMessage(const MidiMessage& message) {};

	///	Sets whether or not the plugin should receive MIDI messages from the MIDI input device.
	void setAcceptMIDI(bool val) {acceptMIDI = val;};
	///	Returns whether or not the plugin should receive MIDI messages from the MIDI input device.
	bool getAcceptMIDI() {return acceptMIDI;};
  protected:
	NPBPluginOrganiser *Parent;
	///	Used by the patch/configuration load/save mechanisms.
	/*!
		This is essentially a type mechanism, used to determine what kind
		of plugin we're loading/saving.  Every subclass of NPBPluginBase has
		it's own ID, determined (IIRC) by it's place in the Organiser's 
		popup menu.
	 */
	int ID;

	///	To let the PluginOrganiser know which plugin needs deleted.
	bool NeedsDeleted;
	///	Host only allows an equal number of input & output channels.
	int NumChannels;

	NPBPluginOrganiser *Organiser;

	///	Whether or not the plugin accepts MIDI messages from the MIDI input device.
	bool acceptMIDI;
};

#endif
